@@ -26,7 +26,7 @@ from __future__ import absolute_import
 import os
 import sys
 
-from thriftpy import load
+from thriftpy2 import load
 
 # dynamically load the thrift modules
 thrift_dir = os.path.join(os.path.dirname(__file__), 'thrift')
@@ -40,46 +40,67 @@ sys.modules[RecordService.__name__] = RecordService
 # import the thrift objects
 from RecordService import (
     TAccessPermissionLevel,
+    TAccessPermissionScope,
     TGetDatabasesParams, TGetTablesParams, TPlanRequestParams,
     TExecTaskParams, TFetchParams, TNetworkAddress, TRecordFormat, TTypeId,
     TGetPartitionsParams,
     TRecordServiceException, TRequestType, RecordServiceWorker,
     # Internal-only exports (used by rest-server)
     TAttribute,
+    TAttributeMatchLevel,
     TAttributeValue,
     TAttributeValueType,
     TDataRegConnection,
     TDelegationToken,
     TErrorCode,
+    TExprType,
+    TInitApplicationParams,
     TObjectOpType,
     TJDBCDataset,
     TCrawlerDiscoveryDataset,
     TCrawlStatus,
     TCrawlerOpType,
-    TRecordServiceException)
+    TPrivilegeType,
+    TRecordServiceException,
+    TSortColOrder,
+    TRequestContext)
 from OkeraRecordService import (
-    OkeraRecordServicePlanner, TExecDDLParams,
+    OkeraRecordServicePlanner, TExecDDLParams, TExecDDLResult,
     # Internal-only exports (used by rest-server)
     TAssignAttributesParams,
+    TAuditQueryParams,
+    TAuthorizeQueryClient,
     TAuthorizeQueryParams,
+    TCatalogObjectType,
+    TCompoundOp,
     TConfigDeleteParams,
     TConfigUpsertParams,
     TConfigType,
+    TCrawlerParams,
     TCreateAttributesParams,
+    TDataRegConnectionParams,
     TDeleteAttributesParams,
+    TDiscoverCrawlerParams,
+    TDiscoverCrawlerResult,
     TGetAccessPermissionsParams,
     TGetAttributeNamespacesParams,
     TGetAttributesParams,
     TGetDatasetsParams,
-    TGetRegisteredObjectsParams,
-    TGetRoleProvenanceParams,
     TGetGrantableRolesParams,
+    TGetInfoParams,
+    TGetRegisteredObjectsParams,
+    TGetRoleParams,
+    THistoryEvent,
+    TGetRoleProvenanceParams,
+    TGetUserAttributesParams,
+    TInfoType,
+    TKvStoreGetParams,
+    TKvStorePutParams,
+    TListCatalogsParams,
+    TListDatabasesParams,
     TListFilesOp,
     TListFilesParams,
-    TDataRegConnectionParams,
-    TDiscoverCrawlerParams,
-    TDiscoverCrawlerResult,
-    TCrawlerParams,
+    TListRolesParams,
     TSetAttributesParams,
     TUnassignAttributesParams,
     TUpdateAttributeMappingsParams)
