@@ -8,11 +8,14 @@
 # pylint: disable=unidiomatic-typecheck
 # pylint: disable=bare-except
 
+import base64
 import datetime
 import json
 import math
 import os
 import random
+import re
+import sqlparse
 import statistics
 import string
 import time
@@ -23,6 +26,7 @@ import pg8000
 from prettytable import PrettyTable
 import pytest
 import pytz
+import sqlparse
 
 from okera import context, _thrift_api
 from okera._thrift_api import TAuthorizeQueryClient
@@ -73,12 +77,51 @@ RUN_PERF = get_env_var('RUN_PERF', bool, False)
 SKIP_PERF_DATA_LOAD = get_env_var('SKIP_PERF_DATA_LOAD', bool, False)
 DEFAULT_ITERS = get_env_var('PERF_ITERATIONS', int, 50)
 QUIET = get_env_var("PERF_QUIET", bool, False)
+# If true, run the snowflake proxy perf as well
+RUN_PERF_SNOWFLAKE = get_env_var("RUN_PERF_SNOWFLAKE", bool, False)
 
 def test_level_lt(min_level):
     if TEST_LEVEL not in TEST_LEVELS:
         return False
     return TEST_LEVELS[TEST_LEVEL] < TEST_LEVELS[min_level]
 
+
+# List of all known CI test groups (see get_ci_test_cases()). Used to
+# validate input.
+ALL_ADDITIONAL_CI_TEST_GROUPS = ['redshift-cte', 'snowflake-cte']
+def get_ci_test_cases(msg):
+    # This parses the gerrit commit message so that additional tests can be run
+    # that are not part of standard precommit. This is useful, for example, when
+    # changing something in the code that needs additional coverage that is normally
+    # run at higher CI.
+    # Input should be base64 encoded message (that gerrit produces normally)
+    # This parses out for CI_TEST_CASES: [comma list of cases]
+    if not msg:
+        return []
+    m = base64.b64decode(msg).decode('ascii')
+    result = []
+    for l in m.split('\n'):
+        if l.startswith('ADDITIONAL_CI_TESTS:'):
+            cases = l.split('ADDITIONAL_CI_TESTS:')[1]
+            for c in cases.split(','):
+                if c:
+                    c = c.strip()
+                    if c not in ALL_ADDITIONAL_CI_TEST_GROUPS:
+                        raise Exception("Invalid CI test group.")
+                    result.append(c)
+    return result
+
+ADDITIONAL_CI_TEST_GROUPS = get_ci_test_cases(
+    get_env_var('GERRIT_CHANGE_COMMIT_MESSAGE', identity, None))
+print('Additional CI Tests: %s' % ADDITIONAL_CI_TEST_GROUPS)
+
+def should_skip(skip_levels, test_group):
+    if test_group not in ALL_ADDITIONAL_CI_TEST_GROUPS:
+        raise Exception("Invalid CI test group.")
+    if test_group in ADDITIONAL_CI_TEST_GROUPS:
+        return False
+    return TEST_LEVEL in skip_levels
+
 def get_test_context(auth_mech=None, dialect=None, namespace='okera', tz=pytz.utc):
     if auth_mech is None:
         auth_mech = get_env_var('PYCEREBRO_TEST_AUTH_MECH', identity, 'NOSASL')
@@ -294,6 +337,9 @@ class PerfResults():
     def run_perf(self):
         return RUN_PERF
 
+    def run_perf_snowflake(self):
+        return RUN_PERF and RUN_PERF_SNOWFLAKE
+
     def persist_results(self, globals):
         print("Persisting results to database.")
         id = globals['id']
@@ -596,7 +642,11 @@ class TestBase(unittest.TestCase):
         conn.execute_ddl("CREATE DATABASE %s" % db)
 
     @staticmethod
-    def _create_all_types(conn, db, include_date=False):
+    def _create_all_types(conn, db,
+                          include_date=False,
+                          include_array=False,
+                          include_map=False,
+                          include_struct=False):
         ddl = '''
             CREATE EXTERNAL TABLE %s.alltypes(
               bool_col BOOLEAN,
@@ -615,6 +665,15 @@ class TestBase(unittest.TestCase):
         ddl += '''
               timestamp_col TIMESTAMP,
               decimal_col decimal(24, 10)
+        '''
+        if include_array:
+            ddl += ',array_col ARRAY<INT>'
+        if include_map:
+            ddl += ',map_col MAP<STRING,INT>'
+        if include_struct:
+            ddl += ',struct_col STRUCT<f1:STRING,f2:INT>'
+
+        ddl +='''
             )
             ROW FORMAT DELIMITED FIELDS TERMINATED BY '|'
             STORED AS TEXTFILE
@@ -913,7 +972,30 @@ class TestBase(unittest.TestCase):
                                  json.dumps(expected, indent=2, sort_keys=True))
         return True
 
-    def authorize_query(self, conn, query, user=None, use_tmp_tables=False, client=None,
+    @staticmethod
+    def normalize_sql(sql):
+        sql = re.sub(' +', ' ', sql)
+        return " ".join(sql.split())
+
+    def assert_sql_equals(self, expected, result, error_header=''):
+        expected = self.normalize_sql(expected)
+        result = self.normalize_sql(result)
+        if expected != result:
+            self.maxDiff = None
+            msg='\n' + error_header
+            msg += "--------EXPECTED---------\n"
+            msg += expected + '\n'
+            msg += "--------RESULT---------\n"
+            msg += result + '\n'
+            msg += "--------EXPECTED (formatted) ---------\n"
+            msg += sqlparse.format(expected, reindent=True) + '\n'
+            msg += "--------RESULT (formatted) ---------\n"
+            msg += sqlparse.format(result, reindent=True) + '\n'
+            print(msg)
+            self.assertEqual(expected, result, msg=msg)
+
+    def authorize_query(self, conn, query, user=None, use_tmp_tables=False,
+                        client=TAuthorizeQueryClient.OKERA,
                         return_full_result=False, cte=False, token=None):
         request = TAuthorizeQueryParams()
         request.sql = query
@@ -929,13 +1011,105 @@ class TestBase(unittest.TestCase):
         if client == TAuthorizeQueryClient.TEST_ACK:
             return ''
         if return_full_result:
-            return ' '.join(result.result_sql.split())
+            return self.normalize_sql(result.result_sql)
         self.assertTrue(result.table is None)
         if result.requires_worker:
             return None
         self.assertTrue(result.result_schema is not None or cte)
-        return ' '.join(result.result_sql.split())
+        return self.normalize_sql(result.result_sql)
+
+    def cte_rewrite(self, conn, query, user=None,
+                    generate_plan=False,
+                    client=TAuthorizeQueryClient.OKERA,
+                    default_db=None,
+                    default_namespace=None,
+                    validate_rewrite=False):
+        request = TAuthorizeQueryParams()
+        request.sql = query
+        request.requesting_user = user
+        request.cte_rewrite = True
+        request.validate_rewrite = validate_rewrite
+        request.plan_request = generate_plan
+        request.client = client
+        request.default_db = default_db
+        request.default_namespace = default_namespace
+        result = conn.service.client.AuthorizeQuery(request)
+        if result.result_sql is None:
+            return None, None
+        result_sql = self.normalize_sql(result.result_sql)
+        return result_sql, result.plan, result.referenced_tables, result.jdbc_referenced_tables
+
+    def verify_tbl_access(self, conn, db, tbl, num_cols, has_db_access=False,
+                            skip_metadata_check=False, timeout=0):
+        """ Verifies the current connect has access to num_cols on this table
+
+            FIXME(BUG): skip_metadata_check should be removed (and always True). It is
+            skipping due to existing bugs.
+        """
+        if num_cols == 0:
+            for ddl in ['describe %s.%s', 'describe formatted %s.%s']:
+                with self.assertRaises(_thrift_api.TRecordServiceException) as ex_ctx:
+                    print(conn.execute_ddl(ddl % (db, tbl)))
+                self.assertTrue('does not have privilege' in str(ex_ctx.exception))
+
+            if has_db_access:
+                # User has access to DB, make sure table does not show up in list
+                self.assertTrue('%s.%s' %(db, tbl) not in conn.list_dataset_names(db))
+                names = conn.list_dataset_names(db)
+                self.assertFalse('%s.%s' %(db, tbl) in names, msg=str(names))
+            else:
+                # Doesn't have database access, ensure listing tables in it fails
+                with self.assertRaises(_thrift_api.TRecordServiceException) as ex_ctx:
+                    print("Listing datasets in: " + db)
+                    print(conn.list_dataset_names(db))
+                self.assertTrue('does not have privilege' in str(ex_ctx.exception))
+
+                datasets = conn.list_datasets(db)
+                self.assertEqual(len(datasets), 0, msg=str(datasets))
+
+                for ddl in ['describe database %s', 'show tables in %s']:
+                    with self.assertRaises(_thrift_api.TRecordServiceException) as ex_ctx:
+                        print(conn.execute_ddl(ddl % db))
+                    self.assertTrue('does not have privilege' in str(ex_ctx.exception))
+
+                if not skip_metadata_check:
+                    dbs = conn.list_databases()
+                    self.assertFalse(db in dbs, msg=str(dbs))
+        else:
+            dbs = conn.list_databases()
+            self.assertTrue(db in dbs, msg=str(dbs))
+            names = conn.list_dataset_names(db)
+            self.assertTrue('%s.%s' %(db, tbl) in names, msg=str(names))
+            datasets = conn.list_datasets(db, name=tbl)
+            self.assertEqual(len(datasets), 1)
+            cols = self._visible_cols(conn.list_datasets(db, name=tbl)[0].schema.cols)
+            self.assertEqual(len(cols), num_cols)
+
+            datasets = conn.list_datasets(db)
+            self.assertTrue(len(datasets) >= 1, msg=str(datasets))
+
+            # Now do a select star and verify the right number of columns are expanded
+            start = time.perf_counter()
+            schema = conn.plan('SELECT * FROM %s.%s' % (db, tbl)).schema
+            end = time.perf_counter()
+            if schema.nested_cols:
+                cols = schema.nested_cols
+            else:
+                cols = schema.cols
+            self.assertEqual(self._top_level_columns(cols), num_cols, msg=str(cols))
+            if timeout > 0:
+                self.assertLess(end - start, timeout)
 
 
 def random_string(length):
     return ''.join(random.choice(string.ascii_uppercase) for _ in range(length))
+
+def create_s3_repro(conn, bucket, key):
+    """ Loads a test case generated from tools/create-repro """
+    import boto3
+    s3 = boto3.client('s3')
+    res = s3.get_object(Bucket=bucket, Key=key)
+    sql = res['Body'].read().decode('UTF-8').strip(' \t\n\r')
+    statements = sqlparse.split(sql)
+    for stmt in statements:
+        conn.execute_ddl(stmt)
