@@ -352,6 +352,63 @@ class AttributesTest(common.TestBase):
             attrs = self.collect_column_attributes(ds)
             self.assertTrue('%s.test.email' % db in attrs)
 
+    def test_autotag_nested_struct(self):
+        db = 'attributes_test_db'
+        ctx = common.get_test_context()
+        with common.get_planner(ctx) as conn:
+            conn.execute_ddl('DROP DATABASE IF EXISTS %s CASCADE' % db)
+            conn.execute_ddl('CREATE DATABASE %s' % db)
+            conn.execute_ddl("""CREATE EXTERNAL TABLE %s.test(
+                `_id` STRING,
+                `app_key` STRING,
+                `app_url` STRING,
+                `created_at` STRING,
+                `features` ARRAY<STRUCT<`_id`: STRING, `approval_text`: STRING,
+                    `auto_sync`: STRING, `code`: STRING, `date_sort`: STRING,
+                    `dont_show_explanation_modal`: STRING, `hash_tag`: STRING,
+                    `kaltura_v2`: STRING, `ks`: STRING, `latest_player`: STRING,
+                    `max_albums`: STRING, `max_galleries`: STRING,
+                    `max_images_per_album`: STRING, `message`: STRING,
+                    `metadata_profile_id`: STRING, `name`: STRING,
+                    `new_image_notification_type`: STRING,
+                    `notifications_email`: STRING, `partner_id`: STRING,
+                    `player_id`: STRING, `player_version`: STRING,
+                    `product_albums_migrator_completed`: STRING,
+                    `search_product_albums_on_elastic`: STRING,
+                    `secondary_sort`: STRING,
+                    `settings`: STRUCT<
+                        `cta`: STRUCT<`enabled`: STRING, `text`: STRING,
+                            `color`: STRING>,
+                        `header`: STRING, `hide_sales`: STRING,
+                        `only_account_images`: STRING,
+                        `promoted_products`: STRUCT<`enabled`: STRING, `text`: STRING>,
+                        `sort`: STRING,
+                        `star_rating`: STRUCT<`enabled`: STRING, `color`: STRING>,
+                        `title`: STRUCT<`enabled`: STRING, `text`: STRING>>,
+                    `status`: STRUCT<`code`: STRING, `message`: STRING>,
+                    `thank_you_needed`: STRING, `thank_you_text`: STRING,
+                    `time_interval_to_save_media`: STRING, `uploader_name`: STRING,
+                    `use_product_service`: STRING>>,
+                `instagram_business_id` STRING,
+                `kaltura_yotpo_partner` STRUCT<`_id`: STRING, `partner_id`: STRING,
+                    `admin_secret`: STRING>,
+                `slug` STRING,
+                `slug_display` STRING,
+                `syndication_pairs` ARRAY<STRUCT<`_id`: STRING, `active`: STRING,
+                    `created_at`: STRING, `destination_account_app_key`: STRING,
+                    `is_mirror`: STRING, `message_timestamp`: STRING,
+                    `syndicate_unmatched`: STRING, `updated_at`: STRING>>,
+                `tokens` ARRAY<STRING>,
+                `updated_at` STRING,
+                `user` STRUCT<`_id`: STRING, `username`: STRING,
+                    `profile_picture`: STRING, `user_id`: STRING>)
+            STORED AS JSON
+            LOCATION 's3://cerebro-customers/yotpo/zd1851/attempt1/'
+            """ % (db))
+
+            # Ensure this does not fail
+            conn.execute_ddl('ALTER TABLE %s.test EXECUTE AUTOTAG' % (db))
+
     def test_cascade(self):
         db = 'attributes_test_db'
         ctx = common.get_test_context()
