@@ -27,7 +27,7 @@ from okera._thrift_api import TAccessPermissionLevel, TRecordServiceException
 
 # The timestamp values are TZ specific, switch based on this.
 # TODO: this is an awful way to do this.
-TIME_ZONE = get_localzone().zone
+TIME_ZONE = str(get_localzone())
 
 DEFAULT_PRESTO_PORT = os.environ['ODAS_TEST_PORT_PRESTO_COORD_HTTPS']
 
