@@ -95,6 +95,13 @@ class DataRegistrationTest(unittest.TestCase):
             conn.execute_ddl('DROP DATACONNECTION IF EXISTS mssql_test_connection')
             conn.execute_ddl('DROP DATACONNECTION IF EXISTS snwflk_test_connection')
 
+    @staticmethod
+    def _contains_attribute(namespace, key, attributes):
+        for attr in attributes:
+            if attr.attribute_namespace == namespace and attr.key == key:
+                return True
+        return False
+
     def _get_psqltest_data_reg_connection_obj(self, name):
         data_reg_connection = TDataRegConnection()
         data_reg_connection.name = name
@@ -507,6 +514,16 @@ class DataRegistrationTest(unittest.TestCase):
             drcs = conn.manage_data_reg_connection("GET", drc)
             self.assertTrue(len(drcs.connections) == 0)
 
+            # create data connection test with whitespace in user key and password key
+            conn.execute_ddl('DROP DATACONNECTION IF EXISTS test_connection_with_whitespace')
+            drc = self._get_psqltest_data_reg_connection_obj("test_connection_with_whitespace")
+
+            # add whitespace and verify connection creation
+            drc.username = " awssm://postgres-jdbcpsqltest-username    "
+            drc.password = " awssm://postgres-jdbcpsqltest-password    "
+            drcs = conn.manage_data_reg_connection("CREATE", drc)
+            self.assertTrue(len(drcs.connections) == 1)
+
     def test_drc_testconnection_as_admin(self):
         MYSQL_CXN = 'test_drc_testconnection_as_admin_mysql_conn'
         SNWFLK_CXN = 'test_drc_testconnection_as_admin_snwflk_conn'
@@ -556,9 +573,7 @@ class DataRegistrationTest(unittest.TestCase):
             self.assertTrue(len(drcs2.connections) == 1)
             self.assertTrue(drcs2.connections[0].test_status == "FAILED")
             self.assertEqual(drcs2.connections[0].test_error_details,
-                "com.cloudera.impala.common.AnalysisException: JDBC data source " \
-                "connection validation failed. Please check connection properties." \
-                "\nError from 'mysql' data source.")
+                "UnknownHostException: invalid_mysql_host: Name or service not known")
             self.assertTrue(drcs2.connections[0].tested_at is not None)
 
             # Test Connection in "EDIT WORKFLOW" with invalid connection details, TEST should fail
@@ -572,9 +587,7 @@ class DataRegistrationTest(unittest.TestCase):
             self.assertTrue(len(drcs2.connections) == 1)
             self.assertTrue(drcs2.connections[0].test_status == "FAILED")
             self.assertEqual(drcs2.connections[0].test_error_details,
-                "com.cloudera.impala.common.AnalysisException: JDBC data source " \
-                "connection validation failed. Please check connection properties." \
-                "\nError from 'snowflake' data source.")
+                "SnowflakeSQLException: Incorrect username or password was specified.")
             self.assertTrue(drcs2.connections[0].tested_at is not None)
 
             # "TEST EXISTING" connection with valid connection details, TEST should succeed
@@ -606,31 +619,50 @@ class DataRegistrationTest(unittest.TestCase):
     def test_invalid_jdbc_props(self):
         ctx = common.get_test_context()
         with common.get_planner(ctx) as conn:
+            invalid_path = 'some-invalid-path'
+
             # validation for invalid path
             drc = self._get_psqltest_data_reg_connection_obj('test_conn_invalid_path')
-            drc.data_source_path = 's3://some-invalid-path/test.conf'
+            drc.data_source_path = 's3://%s/test.conf' % invalid_path
             try:
                 drcs = conn.manage_data_reg_connection('CREATE', drc)
                 assert drcs
             except TRecordServiceException as ex:
-                assert 'JDBC credentials validation failed for data registration ' \
-                    'connection with name ' + drc.name in str(ex.detail)
+                assert 'Bucket %s does not exist' % invalid_path in str(ex.detail)
 
             drc = self._get_psqltest_data_reg_connection_obj('test_conn_invalid_path')
             drcs = conn.manage_data_reg_connection('CREATE', drc)
             self.assertTrue(len(drcs.connections) == 1)
+
             # validation for invalid path update
-            drc.data_source_path = 's3://some-invalid-path/test.conf'
+            drc.data_source_path = 's3://%s/test.conf' % invalid_path
             try:
                 drcs = conn.manage_data_reg_connection('UPDATE', drc)
                 assert drcs
             except TRecordServiceException as ex:
-                assert 'JDBC credentials validation failed for data registration ' \
-                    'connection with name ' + drc.name in str(ex.detail)
+                assert 'Bucket %s does not exist' % invalid_path in str(ex.detail)
 
             drcs = conn.manage_data_reg_connection('DELETE', drc)
             self.assertTrue(len(drcs.connections) == 0)
 
+            # validation for invalid username
+            drc = self._get_snowflake_data_reg_connection_obj('test_conn_invalid_username')
+            drc.user_key = 'awssm://mysql-username'
+            try:
+                drcs = conn.manage_data_reg_connection('TEST_CREATE', drc)
+                assert drcs
+            except TRecordServiceException as ex:
+                assert 'Incorrect username or password was specified.' in str(ex.detail)
+
+            # validation for invalid port
+            drc = self._get_mysqltest_data_reg_connection_obj('test_conn_invalid_port')
+            drc.port = 8080
+            try:
+                drcs = conn.manage_data_reg_connection('CREATE', drc)
+                assert drcs
+            except TRecordServiceException as ex:
+                assert 'SocketTimeoutException: connect timed out' in str(ex.detail)
+
     def test_discover_crawler_catalog(self):
         ctx = common.get_test_context()
         with common.get_planner(ctx) as conn:
@@ -1627,5 +1659,138 @@ class DataRegistrationTest(unittest.TestCase):
             self.assertTrue(len(drcs.connections) == 1)
             self.assertTrue(drcs.connections[0].name == MYSQL_CXN)
 
+    def test_snowflake_tags(self):
+        """ Test that ensures we can sync tags from SF. This is based on the
+            Nong Tags CI SF notebook (in their UI). """
+        CXN = 'test_sf_tags_connection'
+        DB = 'test_sf_tags_db'
+
+        drc = self._get_snowflake_data_reg_connection_obj(CXN)
+        drc.default_catalog = "TAGS_TEST_DB"
+        drc.default_schema = "PUBLIC"
+        drc.connection_properties = {'defaultDb':'PUBLIC'}
+
+        ctx = common.get_test_context()
+        with common.get_planner(ctx) as conn:
+            # The registration will automatically make them, clean these up in case
+            # they exist
+            conn.delete_attribute('DOMAIN_TYPE', 'table')
+            conn.delete_attribute('domain_type', 'view')
+            conn.delete_attribute('pii_type', 'restricted')
+            conn.delete_attribute('pii_type', 'top secret')
+
+            # Run this twice. We only run the clean up above for tags to test the
+            # the case they already exist
+            for _ in range(0, 1):
+                conn.execute_ddl('DROP DATABASE IF EXISTS %s CASCADE' % DB)
+                conn.execute_ddl('DROP DATACONNECTION IF EXISTS %s' % CXN)
+                drcs = conn.manage_data_reg_connection("CREATE", drc)
+                self.assertTrue(len(drcs.connections) == 1)
+
+                conn.execute_ddl('DROP DATABASE IF EXISTS %s CASCADE' % DB)
+                conn.execute_ddl("""CREATE DATABASE %s DBPROPERTIES(
+                    'okera.connection.name' = '%s',
+                    'supports.tags'='true',
+                    'okera.autotagger.skip'='true')""" % (DB, CXN))
+                conn.execute_ddl('ALTER DATABASE %s LOAD DEFINITIONS()' % DB)
+
+                datasets = conn.list_dataset_names(DB)
+                self.assertTrue('test_sf_tags_db.tag_demo_table' in datasets)
+                self.assertTrue('test_sf_tags_db.tag_demo_view' in datasets)
+
+                # Ensure they got created
+                attrs = conn.list_attributes('domain_type')
+                self.assertTrue(self._contains_attribute('domain_type', 'table', attrs))
+                self.assertTrue(self._contains_attribute('domain_type', 'view', attrs))
+                attrs = conn.list_attributes('pii_type')
+                self.assertTrue(self._contains_attribute('pii_type', 'restricted', attrs))
+                self.assertTrue(self._contains_attribute('pii_type', 'top secret', attrs))
+
+                # Validate the tags
+                ds = conn.list_datasets(DB, name='tag_demo_table')[0]
+
+                # Table
+                attr = ds.attribute_values[0].attribute
+                self.assertEqual('domain_type', attr.attribute_namespace)
+                self.assertEqual('table', attr.key)
+
+                # Col tags (order is non deterministic)
+                attr1 = ds.schema.cols[0].attribute_values[0].attribute
+                attr2 = ds.schema.cols[0].attribute_values[1].attribute
+                if attr1.attribute_namespace == 'domain_type':
+                    self.assertEqual('table', attr1.key)
+                    self.assertEqual('pii_type', attr2.attribute_namespace)
+                    self.assertEqual('restricted', attr2.key)
+                else:
+                    self.assertEqual('domain_type', attr2.attribute_namespace)
+                    self.assertEqual('table', attr2.key)
+                    self.assertEqual('pii_type', attr1.attribute_namespace)
+                    self.assertEqual('restricted', attr1.key)
+                attr = ds.schema.cols[1].attribute_values[0].attribute
+                self.assertEqual('domain_type', attr.attribute_namespace)
+                self.assertEqual('table', attr.key)
+
+                # View
+                ds = conn.list_datasets(DB, name='tag_demo_view')[0]
+                self.assertEqual('domain_type', attr.attribute_namespace)
+                self.assertEqual('table', attr.key)
+
+                # Col tags
+                attr1 = ds.schema.cols[0].attribute_values[0].attribute
+                attr2 = ds.schema.cols[0].attribute_values[1].attribute
+                if attr1.attribute_namespace == 'domain_type':
+                    self.assertEqual('view', attr1.key)
+                    self.assertEqual('pii_type', attr2.attribute_namespace)
+                    self.assertEqual('top secret', attr2.key)
+                else:
+                    self.assertEqual('domain_type', attr2.attribute_namespace)
+                    self.assertEqual('view', attr2.key)
+                    self.assertEqual('pii_type', attr1.attribute_namespace)
+                    self.assertEqual('top secret', attr1.key)
+                attr = ds.schema.cols[1].attribute_values[0].attribute
+                self.assertEqual('domain_type', attr.attribute_namespace)
+                self.assertEqual('view', attr.key)
+
+                # Issue an alter to resync
+                conn.execute_ddl('ALTER DATABASE %s LOAD DEFINITIONS(tag_demo_table)' % DB)
+
+    @unittest.skip("This is not a CI test.")
+    def test_snowflake_tags_demo(self):
+        """ This is not really a test but some of the steps for an e2e demo. Parts of are
+            tested above but this is used to validate the demo is working with changing
+            state on the SF side. """
+        CXN = 'test_sf_tags_connection'
+        DB = 'test_sf_tags_db'
+
+        drc = self._get_snowflake_data_reg_connection_obj(CXN)
+        drc.default_catalog = "OKERA_DEMO"
+        drc.default_schema = "HEALTHCARE"
+        drc.connection_properties = {'defaultDb':'PUBLIC'}
+
+        ctx = common.get_test_context()
+        with common.get_planner(ctx) as conn:
+            # The registration will automatically make them, clean these up in case
+            # they exist
+            conn.delete_attribute('DOMAIN_TYPE', 'table')
+            conn.delete_attribute('domain_type', 'view')
+            conn.delete_attribute('pii_type', 'restricted')
+            conn.delete_attribute('pii_type', 'top secret')
+
+            conn.execute_ddl('DROP DATABASE IF EXISTS %s CASCADE' % DB)
+            conn.execute_ddl('DROP DATACONNECTION IF EXISTS %s' % CXN)
+            drcs = conn.manage_data_reg_connection("CREATE", drc)
+            self.assertTrue(len(drcs.connections) == 1)
+
+            conn.execute_ddl('DROP DATABASE IF EXISTS %s CASCADE' % DB)
+            conn.execute_ddl("""CREATE DATABASE %s DBPROPERTIES(
+                'okera.connection.name' = '%s',
+                'supports.tags'='true',
+                'okera.autotagger.skip'='true')""" % (DB, CXN))
+            conn.execute_ddl('ALTER DATABASE %s LOAD DEFINITIONS(PATIENT)' % DB)
+
+            datasets = conn.list_dataset_names(DB)
+            print(datasets)
+            print(conn.execute_ddl('describe formatted test_sf_tags_db.patient'))
+
 if __name__ == "__main__":
     unittest.main()
