@@ -191,6 +191,112 @@ class FsTest(unittest.TestCase):
                                         'does not have access'):
                 authorize_uri(conn, USER, TListFilesOp.READ, '%sfile2' % URI_BASE)
 
+    # This test assumes the following files exist on S3:
+    # s3://cerebrodata-test/fs_test/file1 --> has attribute abac_uri.attr1
+    # s3://cerebrodata-test/fs_test/file2 --> does not have attribute abac_uri.attr1
+    def test_abac_uri_grant_respects_enabled_flag(self):
+        URI_BASE = 's3://cerebrodata-test/fs_test/'
+        USER = 'fstestuser'
+        ROLE = 'abac_uri_test_role'
+        ATTR = 'abac_uri.attr1'
+
+        ctx = common.get_test_context()
+        with common.get_planner(ctx) as conn:
+            ctx.disable_auth()
+
+            ddls = [
+                "DROP ROLE IF EXISTS %s" % (ROLE),
+                "DROP ATTRIBUTE IF EXISTS %s" % (ATTR),
+
+                "CREATE ATTRIBUTE %s" % (ATTR),
+
+                # Create the role and grant it to the user
+                "CREATE ROLE %s" % (ROLE),
+                "GRANT ROLE %s TO GROUP %s" % (ROLE, USER),
+
+                """
+                GRANT ALL ON URI '%s'
+                HAVING ATTRIBUTE IN (%s)
+                TO ROLE %s POLICYPROPERTIES('enabled'='false')
+                """ % (URI_BASE, ATTR, ROLE),
+            ]
+
+            for ddl in ddls:
+                conn.execute_ddl(ddl)
+
+            # User should not have access because the grant is disabled
+            with self.assertRaisesRegex(_thrift_api.TRecordServiceException,
+                                        'does not have access'):
+                authorize_uri(conn, USER, TListFilesOp.READ, '%sfile1' % URI_BASE)
+            # User should not have access because the file doesn't have the tag
+            with self.assertRaisesRegex(_thrift_api.TRecordServiceException,
+                                        'does not have access'):
+                authorize_uri(conn, USER, TListFilesOp.READ, '%sfile2' % URI_BASE)
+
+            conn.execute_ddl("""
+                REVOKE ALL ON URI '%s'
+                HAVING ATTRIBUTE IN (%s)
+                FROM ROLE %s
+                """ % (URI_BASE, ATTR, ROLE))
+            conn.execute_ddl("""
+                GRANT ALL ON URI '%s'
+                HAVING ATTRIBUTE IN (%s)
+                TO ROLE %s
+                POLICYPROPERTIES('enabled'='true')
+                """ % (URI_BASE, ATTR, ROLE))
+            # Grant is now enabled so should have access
+            authorize_uri(conn, USER, TListFilesOp.READ, '%sfile1' % URI_BASE)
+
+            # File is missing tag so still don't have access
+            with self.assertRaisesRegex(_thrift_api.TRecordServiceException,
+                                        'does not have access'):
+                authorize_uri(conn, USER, TListFilesOp.READ, '%sfile2' % URI_BASE)
+
+    def test_rbac_uri_grant_respects_enabled_flag(self):
+        URI_BASE = 's3://cerebrodata-test/nytaxi-data'
+        USER = 'fstestuser'
+        ROLE = 'rbac_uri_test_role'
+
+        ctx = common.get_test_context()
+        with common.get_planner(ctx) as conn:
+            ctx.disable_auth()
+
+            ddls = [
+                "DROP ROLE IF EXISTS %s" % (ROLE),
+
+                # Create the role and grant it to the user
+                "CREATE ROLE %s" % (ROLE),
+                "GRANT ROLE %s TO GROUP %s" % (ROLE, USER),
+
+                """
+                GRANT SELECT ON URI '%s/csv'
+                TO ROLE %s
+                POLICYPROPERTIES('enabled'='false')
+                """ % (URI_BASE, ROLE)
+            ]
+
+            for ddl in ddls:
+                conn.execute_ddl(ddl)
+
+            # User should not have access because the grant is disabled
+            with self.assertRaisesRegex(_thrift_api.TRecordServiceException,
+                                        'does not have access'):
+                authorize_uri(conn, USER, TListFilesOp.READ,
+                      '%s/csv/trips_xcp.csv.gz' % URI_BASE)
+
+            conn.execute_ddl("""
+                REVOKE ALL ON URI '%s'
+                FROM ROLE %s
+                """ % (URI_BASE, ROLE))
+            conn.execute_ddl("""
+                GRANT ALL ON URI '%s'
+                TO ROLE %s
+                POLICYPROPERTIES('enabled'='true')
+                """ % (URI_BASE, ROLE))
+            # Grant is now enabled so should have access
+            authorize_uri(conn, USER, TListFilesOp.READ,
+                      '%s/csv/trips_xcp.csv.gz' % URI_BASE)
+
     def test_ls(self):
         ctx = common.get_test_context()
         planner = common.get_planner(ctx)
