@@ -81,7 +81,7 @@ class JdbcScanTest(unittest.TestCase):
             # The 'test' db exists in this dremio cluster but is empty
             for create in create_connection:
                 conn.execute_ddl('DROP DATABASE IF EXISTS %s CASCADE' % DB)
-                conn.execute_ddl('DROP DATACONNECTION %s' % CXN)
+                conn.execute_ddl('DROP DATACONNECTION IF EXISTS %s' % CXN)
                 conn.execute_ddl(create)
                 conn.execute_ddl("""CREATE DATABASE %s DBPROPERTIES(
                     'okera.connection.name' = '%s',
@@ -329,3 +329,4 @@ class JdbcScanTest(unittest.TestCase):
                 db_name='jdbc_saphana_test', credentials_file=cred_file,
                 schema_name=s_name, expected_table_count=count, skip_scan=skip,
                 ignore_tables_list=ignore_tables)
+
