@@ -4,6 +4,7 @@
 #
 # pylint: disable=line-too-long
 # pylint: disable=too-many-locals
+# pylint: disable=duplicate-code
 
 from datetime import datetime, timedelta
 import decimal
@@ -16,16 +17,14 @@ import unittest
 import boto3
 import requests
 
-from okera import context
+from okera import _thrift_api
 from okera.tests import pycerebro_test_common as common
-from okera._thrift_api import TRecordServiceException
+from okera.tests import file_format_test_common as fileFormat
+from okera._thrift_api import TTypeId
 
 TEST_DB = 'json_test_db'
 TEST_TABLE = TEST_DB + ".tbl"
-DEFAULT_PRESTO_PORT = 8049
 DEFAULT_ENABLE_PRESTO = False
-DEFAULT_ENABLE_SCAN_API = True
-SCAN_API_RECORDS = 100
 
 S3_TEST_FILES = [
     's3://cerebrodata-test/chase/json-empty/chase-json-test.json',
@@ -165,84 +164,14 @@ TEST_ERROR_URLS = [
     #('https://raw.githubusercontent.com/simdjson/simdjson/master/jsonexamples/numbers.json', ''),
 ]
 
+TEST_DB = 'json_test_db'
+TEST_TABLE = TEST_DB + ".tbl"
 S3 = boto3.resource('s3')
 TEST_BUCKET = 'cerebrodata-dev'
 TEST_DIR_PREFIX = 'json-random-testing'
+FILE_FORMAT = 'JSON'
 
-class JsonIntegrationTest(common.TestBase):
-    def __test_path(self, path, expected_json, allow_missing=False, batch_mode=False,
-                    test_presto=DEFAULT_ENABLE_PRESTO, sql=None, required_type=None,
-                    sort_by_record_idx=False, test_scan_api=DEFAULT_ENABLE_SCAN_API):
-        """ Runs schema inference and verification for the json file at path. Returns
-            (SQL, skipped) on failure/skip and None on success.
-        """
-        if not sql:
-            sql = '''
-    CREATE EXTERNAL TABLE %s LIKE JSON '%s'
-    STORED AS JSON
-    LOCATION '%s' ''' % (TEST_TABLE, path, path)
-        presto_port = None
-        if test_presto:
-            presto_port = DEFAULT_PRESTO_PORT
-        ctx = context()
-        ctx.enable_token_auth(token_str='root')
-        with ctx.connect(presto_port=presto_port) as conn:
-            conn.execute_ddl('DROP DATABASE IF EXISTS %s CASCADE' % TEST_DB)
-            conn.execute_ddl('CREATE DATABASE %s' % TEST_DB)
-            try:
-                conn.execute_ddl(sql)
-            except TRecordServiceException as ex:
-                # Skip failures if the schema is longer than 4000
-                if batch_mode and 'exceeds maximum type length' in ex.detail:
-                    return sql, True
-                raise
-            if not batch_mode:
-                print("SQL: %s" % sql)
-            actual_schema = conn.plan(TEST_TABLE).schema
-            # Use max_task_count to make the results stable
-            actual_json_pyokera = conn.scan_as_json(TEST_TABLE, strings_as_utf8=True,
-                                                    max_task_count=1,
-                                                    dialect='okera')
-            if presto_port:
-                actual_json_presto = conn.scan_as_json(
-                    'SELECT * FROM %s' % TEST_TABLE, strings_as_utf8=True,
-                    max_task_count=1, dialect='presto')
-
-            if test_scan_api:
-                url = 'http://localhost:11050/scan/%s?records=%s' % \
-                    (TEST_TABLE, SCAN_API_RECORDS)
-                actual_scan_json = json.loads(requests.get(url).text)
-
-        self.assertTrue(actual_schema is not None)
-        # Verify scanning with pyokera
-        res = self.compare_json(actual_json_pyokera, expected_json, allow_missing,
-                                empty_struct_equals_null=True, batch_mode=batch_mode,
-                                required_type=required_type)
-        if not res:
-            return sql, False
-
-        # Verify presto if enabled
-        if presto_port:
-            if sort_by_record_idx:
-                # Presto return records in a "random" order if the result set is
-                # sufficiently large. Sort them by a key that exists to get comparable
-                # results.
-                actual_json_presto = sorted(actual_json_presto, key=lambda k: k['idx'])
-                expected_json = sorted(expected_json, key=lambda k: k['idx'])
-            res = self.compare_json(actual_json_presto, expected_json, allow_missing,
-                                    empty_struct_equals_null=True, batch_mode=batch_mode,
-                                    required_type=required_type)
-            if not res:
-                return sql, False
-
-        if test_scan_api:
-            res = self.compare_json(actual_scan_json, expected_json[0:SCAN_API_RECORDS],
-                                    allow_missing, empty_struct_equals_null=True,
-                                    batch_mode=batch_mode, required_type=required_type)
-            if not res:
-                return sql, False
-
-        return None, False
+class JsonIntegrationTest(fileFormat.FileFormatTestBase):
 
     def __test_s3(self, path, allow_missing=False, test_scan_api=False):
         # Read it directly using python libraries
@@ -251,14 +180,16 @@ class JsonIntegrationTest(common.TestBase):
         expected_json = []
         for line in obj.get()['Body'].read().decode('utf-8').strip().split('\n'):
             expected_json.append(json.loads(line))
-        self.__test_path(path, expected_json, allow_missing, test_scan_api=test_scan_api)
+        self._test_path(path, expected_json, TEST_DB, TEST_TABLE, allow_missing,
+                        test_scan_api=test_scan_api, file_format=FILE_FORMAT)
 
     def __test_url(self, url, key, expected_json=None):
         print("Testing (%s, %s)" % (url, key))
         expected_json = json.loads(requests.get(url).text)
         if key is None:
             expected_json = [expected_json]
-            self.__test_path(url, expected_json, allow_missing=True)
+            self._test_path(url, expected_json, TEST_DB, TEST_TABLE, allow_missing=True,
+                            file_format=FILE_FORMAT)
             return
 
         # There is a key, we need to transform the data
@@ -270,21 +201,10 @@ class JsonIntegrationTest(common.TestBase):
         data = []
         for record in expected_json:
             data.append(json.dumps(record))
-        path = self._write_s3('json-http-testing/data.json', '\n'.join(data))
-        self.__test_path(path, expected_json, allow_missing=True)
-
-    @staticmethod
-    def _write_s3(path, data):
-        filename = '%s/%s' % (TEST_DIR_PREFIX, path)
-        o = S3.Object(TEST_BUCKET, filename)
-        o.put(Body=data)
-        return 's3://%s/%s' % (TEST_BUCKET, filename)
-
-    @staticmethod
-    def _cleanup(path):
-        bucket = S3.Bucket(TEST_BUCKET)
-        prefix = '%s/%s' % (TEST_DIR_PREFIX, path)
-        bucket.objects.filter(Prefix=prefix).delete()
+        path = self._write_s3(TEST_DIR_PREFIX, 'json-http-testing/data.json',
+                              '\n'.join(data))
+        self._test_path(path, expected_json, TEST_DB, TEST_TABLE, allow_missing=True,
+                        file_format=FILE_FORMAT)
 
     def _run_random_tests(self, generator, iters, path):
         # Run random generator tests
@@ -300,8 +220,9 @@ class JsonIntegrationTest(common.TestBase):
             output = []
             for record in data:
                 output.append(json.dumps(record))
-            full_path = self._write_s3(filename, '\n'.join(output))
-            self.__test_path(full_path, data, allow_missing=True)
+            full_path = self._write_s3(TEST_DIR_PREFIX, filename, '\n'.join(output))
+            self._test_path(full_path, data, TEST_DB, TEST_TABLE, allow_missing=True,
+                            file_format=FILE_FORMAT)
 
     @unittest.skip("Example for use during development to test a specific dataset.")
     def test_single_dataset(self):
@@ -315,6 +236,120 @@ class JsonIntegrationTest(common.TestBase):
         for f in S3_TEST_FILES:
             self.__test_s3(f, allow_missing=True, test_scan_api=True)
 
+    def valid_json_api(self, url):
+        ctx = common.get_test_context()
+        disable_prop = "WITH SERDEPROPERTIES('new.line.json'='true') "
+        with common.get_planner(ctx) as conn:
+            conn.execute_ddl('CREATE DATABASE IF NOT EXISTS %s' % TEST_DB)
+
+            # Explicit schema
+            conn.execute_ddl('DROP TABLE IF EXISTS %s' % TEST_TABLE)
+            conn.execute_ddl(
+                ("CREATE EXTERNAL TABLE %s(revenue BIGINT) STORED AS JSON " +
+                 "LOCATION '%s'") % (TEST_TABLE, url))
+            json = conn.scan_as_json(TEST_TABLE, max_records=2)
+            # TODO: not sure how stable this live endpoint is so making the validation
+            # less strict
+            self.assertEqual(2, len(json))
+            self.assertEqual(1, len(json[0]))
+            self.assertTrue('revenue' in json[0])
+
+            # Explicitly disable new line json, this won't scan right
+            conn.execute_ddl('DROP TABLE %s' % TEST_TABLE)
+            conn.execute_ddl(
+                ("CREATE EXTERNAL TABLE %s(revenue BIGINT) %s " +
+                 "STORED AS JSON " +
+                 "LOCATION '%s'") % (TEST_TABLE, disable_prop, url))
+            json = conn.scan_as_json(TEST_TABLE, max_records=2)
+            self.assertEqual([], json)
+
+            # Infer full schema
+            conn.execute_ddl('DROP TABLE %s' % TEST_TABLE)
+            conn.execute_ddl(
+                ("CREATE EXTERNAL TABLE %s " +
+                 "LIKE JSON '%s' " +
+                 "STORED AS JSON " +
+                 "LOCATION '%s'") % (TEST_TABLE, url, url))
+            json = conn.scan_as_json(TEST_TABLE, max_records=2)
+            self.assertEqual(2, len(json))
+            self.assertTrue(len(json[0]) > 20)
+            self.assertTrue('revenue' in json[0])
+            self.assertTrue('symbol' in json[0])
+
+            # Disable new line json, we won't infer it right
+            conn.execute_ddl('DROP TABLE %s' % TEST_TABLE)
+            with self.assertRaises(_thrift_api.TRecordServiceException) as ex_ctx:
+                conn.execute_ddl(
+                    ("CREATE EXTERNAL TABLE %s " +
+                     "LIKE JSON '%s' %s " +
+                     "STORED AS JSON " +
+                     "LOCATION '%s'") % (TEST_TABLE, url, disable_prop, url))
+            print(ex_ctx.exception)
+            self.assertTrue('Failed to read JSON schema' in str(ex_ctx.exception))
+
+    def test_json_api(self):
+        # this endpoint responds with 403, disabling this one for now
+        url = 'https://financialmodelingprep.com/api/v3/income-statement/AAPL?' +\
+            'limit=10&apikey='
+        # Hardcoded apikey
+
+        # self.valid_json_api(url + 'demo')
+        # Dynamically returned api key
+        # self.valid_json_api(url + '${script:/get-financialmodelingprep-apikey.sh}')
+
+    def test_json_api_path(self):
+        ctx = common.get_test_context()
+        path_prop = "WITH SERDEPROPERTIES('json.records-start.path'='content') "
+        url = 'https://gist.githubusercontent.com/nongli/' +\
+            'd7e30635e466a9352bbcaacfb36a1c09/raw/' +\
+            'fea92d963c724fd2d6e1e71fd2c9cba7daa3e277/gistfile1.txt'
+
+        with common.get_planner(ctx) as conn:
+            conn.execute_ddl('CREATE DATABASE IF NOT EXISTS %s' % TEST_DB)
+
+            conn.execute_ddl('DROP TABLE IF EXISTS %s' % TEST_TABLE)
+            conn.execute_ddl(
+                ("CREATE EXTERNAL TABLE %s " +
+                 "LIKE JSON '%s' %s" +
+                 "STORED AS JSON " +
+                 "LOCATION '%s'") % (TEST_TABLE, url, path_prop, url))
+            json = conn.scan_as_json(TEST_TABLE)
+            self.assertEqual(1, len(json))
+            self.assertEqual(len(json[0]), 5)
+            self.assertEqual(json[0]['bdrid'], '3095')
+            self.assertEqual(json[0]['subtype'], 'OFFICE')
+
+            # Set the config on the DB, it should apply to all tables in it
+            conn.execute_ddl('DROP DATABASE %s CASCADE' % TEST_DB)
+            conn.execute_ddl('CREATE DATABASE %s' % TEST_DB)
+            conn.execute_ddl(
+                ("ALTER DATABASE %s SET DBPROPERTIES(" + \
+                "'json.records-start.path'='content')") % TEST_DB)
+            conn.execute_ddl(
+                ("CREATE EXTERNAL TABLE %s " +
+                 "LIKE JSON '%s'" +
+                 "STORED AS JSON " +
+                 "LOCATION '%s'") % (TEST_TABLE, url, url))
+            json = conn.scan_as_json(TEST_TABLE)
+            self.assertEqual(1, len(json))
+            self.assertEqual(len(json[0]), 5)
+            self.assertEqual(json[0]['bdrid'], '3095')
+            self.assertEqual(json[0]['subtype'], 'OFFICE')
+
+    def test_json_count(self):
+        ctx = common.get_test_context()
+        with common.get_planner(ctx) as conn:
+            data = conn.scan_as_json(
+                'select count(*) as cnt from rs_json_format.airlines_mini',
+                strings_as_utf8=False,
+                min_task_size=5 * 1000)
+            # for this dataset we expect exactly 1 task splits as json is not splittable
+            self.assertTrue(len(data) == 1)
+            sum = 0
+            for cnt in data:
+                sum += cnt['cnt']
+            self.assertEqual(sum, 100)
+
     @unittest.skipIf(common.TEST_LEVEL != "all", "This writes to the same location in s3")
     def test_type_conversion(self):
         # This test does not use schema infernce and instead explicitly creates the
@@ -420,14 +455,16 @@ class JsonIntegrationTest(common.TestBase):
             for _ in range(0, 3):
                 contents.append(json.dumps(data))
 
-            path = self._write_s3('type-conversion/data.json', '\n'.join(contents))
+            path = self._write_s3(TEST_DIR_PREFIX, 'type-conversion/data.json',
+                                  '\n'.join(contents))
             create = '''
         CREATE EXTERNAL TABLE %s(%s)
         STORED AS JSON
         LOCATION '%s' ''' % \
             (TEST_TABLE, ', '.join(cols), path)
-            self.__test_path(path, [record, record, record],
-                             sql=create, required_type=t[1])
+            self._test_path(path, [record, record, record], TEST_DB, TEST_TABLE,
+                            sql=create, required_type=t[1],
+                            file_format=FILE_FORMAT)
 
     @unittest.skipIf(common.TEST_LEVEL != "all",
                      "Not very stable, downloads from internet.")
@@ -437,16 +474,16 @@ class JsonIntegrationTest(common.TestBase):
 
     @unittest.skipIf(common.TEST_LEVEL != "all", "Skipping at unit test level")
     def test_basic_random_schemas(self):
-        self._cleanup('basic/')
+        self._cleanup(TEST_DIR_PREFIX, 'basic/')
         generator = common.JsonGenerator(
             seed=int(time.time() * 1000))
         self._run_random_tests(generator, None, 'basic')
         # Only cleanup on success, this helps with S3 consistency issues.
-        self._cleanup('basic/')
+        self._cleanup(TEST_DIR_PREFIX, 'basic/')
 
     @unittest.skipIf(common.TEST_LEVEL != "all", "Skipping at unit test level")
     def test_basic_schema_merge(self):
-        self._cleanup('merge/')
+        self._cleanup(TEST_DIR_PREFIX, 'merge/')
         generator = common.JsonGenerator(
             seed=int(time.time() * 1000),
             min_records=2,
@@ -458,11 +495,11 @@ class JsonIntegrationTest(common.TestBase):
             generate_variadic_schema=True)
         self._run_random_tests(generator, None, 'merge')
         # Only cleanup on success, this helps with S3 consistency issues.
-        self._cleanup('merge/')
+        self._cleanup(TEST_DIR_PREFIX, 'merge/')
 
     @unittest.skip("Finding too many bugs")
     def test_medium_random_schemas(self):
-        self._cleanup('medium/')
+        self._cleanup(TEST_DIR_PREFIX, 'medium/')
         generator = common.JsonGenerator(
             seed=int(time.time() * 1000),
             null_probability=.15,
@@ -472,7 +509,7 @@ class JsonIntegrationTest(common.TestBase):
             max_recursion=6)
         self._run_random_tests(generator, None, 'medium')
         # Only cleanup on success, this helps with S3 consistency issues.
-        self._cleanup('medium/')
+        self._cleanup(TEST_DIR_PREFIX, 'medium/')
 
     @unittest.skipIf(common.TEST_LEVEL != "all", "Skipping at unit test level")
     def test_long_running(self):
@@ -512,6 +549,27 @@ class JsonIntegrationTest(common.TestBase):
             generate_variadic_schema=True)
         self._test_long_running(generator)
 
+    @unittest.skipIf(common.TEST_LEVEL != "all", "Skipping at unit test level")
+    def test_even_larger_long_running(self):
+        # Runs the test for a fixed amount of time or until the max failures are
+        # hit. This will archive the test files to S3 for async debugging.
+        print("\n\n----------------------------------------------------------------")
+        generator = common.JsonGenerator(
+            types=[[TTypeId.STRING]],
+            seed=int(time.time() * 1000),
+            null_probability=.05,
+            empty_record_probability=0,
+            missing_fields_probability=0,
+            min_fields=1,
+            max_fields=2,
+            max_array_len=5,
+            max_recursion=1,
+            min_records=10000,
+            max_records=50000,
+            generate_empty_record_all_types=False,
+            generate_variadic_schema=False)
+        self._test_long_running(generator)
+
     def _test_long_running(self, generator):
         # Max failures and duration to run
         max_failures = 5
@@ -549,11 +607,12 @@ class JsonIntegrationTest(common.TestBase):
             output = []
             for record in data:
                 output.append(json.dumps(record))
-            full_path = self._write_s3(filename, '\n'.join(output))
+            full_path = self._write_s3(TEST_DIR_PREFIX, filename, '\n'.join(output))
 
-            sql, skip = self.__test_path(
-                full_path, data, allow_missing=True, batch_mode=True,
-                test_presto=test_presto, sort_by_record_idx=test_presto)
+            sql, skip = self._test_path(
+                full_path, data, TEST_DB, TEST_TABLE, allow_missing=True, batch_mode=True,
+                test_presto=test_presto, sort_by_record_idx=test_presto,
+                file_format=FILE_FORMAT)
             if not sql:
                 total_success += 1
                 continue
