@@ -203,7 +203,7 @@ class MindsphereApiTest(common.TestBase):
         request.client = client
         request.request_filter = user_filter
         result = conn.service.client.AuthorizeQuery(request)
-        return ' '.join(result.result_sql.split())
+        return self.normalize_sql(result.result_sql)
 
     def _authorize_assets(self, label, conn, user, data,
                           user_filter=None, measure=False,
@@ -435,7 +435,7 @@ class MindsphereApiTest(common.TestBase):
                 "SELECT name FROM %s.assets" % TEST_DB, user=TEST_USER)
             self.assertEqual(sql,
                 "SELECT name FROM mindsphere_test.assets WHERE " +
-                "`location`.country = 'Austria'")
+                "location.country = 'Austria'")
 
     def test_dax_filter(self):
         ctx = common.get_test_context()
@@ -480,7 +480,7 @@ class MindsphereApiTest(common.TestBase):
                 "SELECT name FROM %s.assets" % TEST_DB, user=TEST_USER)
             self.assertEqual(sql,
                 "SELECT name FROM mindsphere_test.assets WHERE " +
-                "`location`.longitude > 50")
+                "location.longitude > 50")
 
     @pytest.mark.perf
     def test_hierarchy_filter(self):
@@ -1131,7 +1131,7 @@ class MindsphereApiTest(common.TestBase):
             sql = self._authorize_sql(conn,
                 "SELECT * FROM %s.assets" % TEST_DB, user=TEST_USER)
             self.assertEqual(sql,
-                "SELECT name, description, `location` as location, timezone " +
+                "SELECT name, description, location, timezone " +
                 "FROM mindsphere_test.assets")
 
     @pytest.mark.perf
