@@ -26,20 +26,22 @@
 |      get_schema      | Wide table w/ many attrs | many_attrs_user | 442.111 | 438.563 | 451.890 | 503.797 | 503.797 |  100  |
 +----------------------+--------------------------+-----------------+---------+---------+---------+---------+---------+-------+
 """
-
-import os
-import statistics
 import unittest
 import pytest
 
-from okera import context
 from okera import _thrift_api
-from okera._thrift_api import TAuthorizeQueryClient
+from okera._thrift_api import (
+    TAccessPermissionLevel,
+    TAccessPermissionScope,
+    TAttributeMatchLevel,
+    TAuthorizeQueryClient,
+    TPlanRequestParams,
+    TRequestType
+)
 from okera.tests import create_test_data
 from okera.tests import pycerebro_test_common as common
 
-from okera._thrift_api import (TInitApplicationParams, TPlanRequestParams, TRequestType,
-    TAttribute, TAttributeMatchLevel, TAccessPermissionLevel)
+from sfproxy_test import get_snowflake_proxy_test_context, get_snowflake_proxy
 
 ATTR = "perf_test.attr1"
 DB = "python_perf_test_db"
@@ -119,6 +121,133 @@ class PerfTest(common.TestBase):
                 conn, MANY_ATTRS_DB, MANY_ATTRS_TABLE, MANY_ATTRS_ROLE, MANY_ATTRS_USER)
         print("...Done setting up perf test")
 
+    @unittest.skipIf(not PERF.run_perf(), "Skipping perf tests")
+    def test_list_roles(self):
+        ctx = common.get_test_context()
+        with common.get_planner(ctx) as conn:
+            print("...Setting up list_roles test")
+            total_roles = 150
+            role_prefix = "list_roles_test"
+            user_prefix = "list_roles_user"
+            for i in range(total_roles):
+                role_name = role_prefix + str(i)
+                user_name = user_prefix + str(i)
+                conn.execute_ddl("CREATE ROLE IF NOT EXISTS %s WITH GROUPS %s" % (role_name, user_name))
+                conn.execute_ddl("""
+                GRANT IF NOT EXISTS SELECT ON DATABASE %s
+                HAVING ATTRIBUTE IN (%s)
+                TO ROLE %s
+                POLICYPROPERTIES('enabled'='true', 'start_datetime'='543453453', 'end_datetime'='543453459')
+                """ % (DB, ATTR, role_name))
+                conn.execute_ddl("""
+                GRANT IF NOT EXISTS SELECT ON TABLE %s.%s
+                TO ROLE %s
+                POLICYPROPERTIES('enabled'='true')
+                """ % (DB, TBL1, role_name))
+
+            def list(t_params, expected_results):
+                def get():
+                    t_params.return_structured_exprs = True
+                    t_params.limit = 10000
+                    roles = conn._underlying_client().ListRoles(t_params)
+                    assert len(roles.roles) == expected_results
+                return get
+
+            print("...starting perf run for list_roles")
+
+            t_params = _thrift_api.TListRolesParams()
+            PERF.measure(list(t_params, total_roles + 7), "Listing Roles",
+                         "admin", "ListRoles")
+
+            t_params = _thrift_api.TListRolesParams()
+            t_params.scopes = [TAccessPermissionScope.DATABASE]
+            PERF.measure(list(t_params, total_roles + 3), "Listing Roles w/scope filter",
+                         "admin", "ListRoles")
+
+            t_params = _thrift_api.TListRolesParams()
+            t_params.group_names = [
+                "list_roles_user1",
+                "list_roles_user125",
+                "list_roles_user10",
+                "no_match"]
+            PERF.measure(list(t_params, 3), "Listing Roles w/group_names filter",
+                         "admin", "ListRoles")
+
+            t_params = _thrift_api.TListRolesParams()
+            t_params.role_name_substring = role_prefix
+            PERF.measure(list(t_params, total_roles), "Listing Roles w/role name substring filter",
+                         "admin", "ListRoles")
+
+    @unittest.skipIf(not PERF.run_perf_snowflake(), "Skipping snowflake perf tests")
+    def test_measure_snowflake_speed_of_light(self):
+        sf_ctx = get_snowflake_proxy_test_context()
+
+        setup_ddls_params = {
+            'conn' : 'snowflake_connection_demo',
+            'group' : sf_ctx.username,
+            'role' : 'test_perf_sf_role',
+            'db' : 'test_perf_sf',
+        }
+        setup_ddls = [
+            "DROP DATABASE IF EXISTS {db} CASCADE INCLUDING PERMISSIONS",
+            "CREATE DATABASE {db}",
+            """
+              CREATE DATACONNECTION IF NOT EXISTS {conn} CXNPROPERTIES
+                (
+                  'connection_type' = 'JDBC',
+                  'driver' = 'snowflake',
+                  'account' = 'vq85960',
+                  'host' = 'vq85960.snowflakecomputing.com',
+                  'port'= '443',
+                  'user'= 'awssm://okera/demo/snowflake/username',
+                  'password' = 'awssm://okera/demo/snowflake/password',
+                  'jdbc.db.name' = 'OKERA_DEMO'
+                )""",
+            """
+              CREATE EXTERNAL TABLE {db}.hospital_discharge
+              STORED AS JDBC
+              TBLPROPERTIES(
+                'driver' = 'snowflake',
+                'okera.connection.name' = '{conn}',
+                'jdbc.db.name' = 'OKERA_DEMO',
+                'jdbc.schema.name' = 'HEALTHCARE',
+                'table' = 'HOSPITAL_DISCHARGE'
+              )
+            """,
+            """
+              CREATE EXTERNAL TABLE {db}.patient
+              STORED AS JDBC
+              TBLPROPERTIES(
+                'driver' = 'snowflake',
+                'okera.connection.name' = '{conn}',
+                'jdbc.db.name' = 'OKERA_DEMO',
+                'jdbc.schema.name' = 'HEALTHCARE',
+                'table' = 'PATIENT'
+              )
+            """,
+            "DROP ROLE IF EXISTS {role}",
+            "CREATE ROLE {role}",
+            "GRANT ROLE {role} TO GROUP {group}",
+            "GRANT SELECT ON DATABASE {db} TO ROLE {role}",
+        ]
+        planner_ctx = common.get_test_context()
+        with common.get_planner(planner_ctx) as conn:
+            for ddl_tmpl in setup_ddls:
+                ddl = ddl_tmpl.format(**setup_ddls_params)
+                conn.execute_ddl(ddl)
+
+        def conn_query(conn):
+            def get():
+                res = conn.query_request(
+                    "SELECT * FROM OKERA_DEMO.HEALTHCARE.PATIENT LIMIT 1")
+                assert len(res.get('rowset')) == 1
+            return get
+
+        with get_snowflake_proxy(sf_ctx) as conn:
+            conn.query_request("USE ROLE {}".format(sf_ctx.user_role))
+            PERF.measure(conn_query(conn), "sfproxy: speed of light",
+                        sf_ctx.username, "sfproxy_query_request")
+
     @unittest.skipIf(not PERF.run_perf(), "Skipping perf tests")
     def test_wide_table_with_attributes(self):
         ctx = common.get_test_context()
@@ -402,11 +531,6 @@ class PerfTest(common.TestBase):
     # TListDatabasesResult ListDatabases(1: TListDatabasesParams params)
     #    throws(1:RecordService.TRecordServiceException ex);
     #
-    # TListRolesResult ListRoles(1: TListRolesParams params)
-    #    throws(1:RecordService.TRecordServiceException ex);
-    # TGetRoleResult GetRole(1: TGetRoleParams params)
-    #    throws(1:RecordService.TRecordServiceException ex);
-    #
     # TKvStoreGetResult GetFromKvStore(1: TKvStoreGetParams params)
     #    throws(1:RecordService.TRecordServiceException ex);
     # TKvStorePutResult PutToKvStore(1: TKvStorePutParams params)
