@@ -9,6 +9,7 @@
 
 import unittest
 
+from okera._thrift_api import TAuthorizeQueryClient
 from okera._thrift_api import TAuthorizeQueryParams
 from okera.tests import pycerebro_test_common as common
 
@@ -212,8 +213,9 @@ def authorize_query(conn, query, user):
     request = TAuthorizeQueryParams()
     request.sql = query
     request.requesting_user = user
+    request.client = TAuthorizeQueryClient.OKERA
     result = conn.service.client.AuthorizeQuery(request)
-    return ' '.join(result.result_sql.split())
+    return common.TestBase.normalize_sql(result.result_sql)
 
 class PrivacyTest(unittest.TestCase):
     def _validate_query(self, ctx, conn, query,
@@ -477,6 +479,7 @@ class PrivacyTest(unittest.TestCase):
     def test_nfp_ref_tokenize(self):
         self._test_fn('nfp_ref_tokenize', maintains_ref_integrity=True, stable=True, maintains_type=False)
 
+    @unittest.skipIf(common.test_level_lt('checkin'), "Skipping below checkin test level")
     def test_nfp_noref_tokenize(self):
         self._test_fn('nfp_noref_tokenize', maintains_ref_integrity=False, stable=True, maintains_type=False)
 
@@ -484,6 +487,7 @@ class PrivacyTest(unittest.TestCase):
     def test_fp_ref_tokenize(self):
         self._test_fn('fp_ref_tokenize', maintains_ref_integrity=True, stable=True, maintains_type=True)
 
+    @unittest.skipIf(common.test_level_lt('checkin'), "Skipping below checkin test level")
     def test_fp_noref_tokenize(self):
         self._test_fn('fp_noref_tokenize', maintains_ref_integrity=False, stable=True, maintains_type=True)
 
@@ -506,6 +510,7 @@ class PrivacyTest(unittest.TestCase):
     def test_fp_random(self):
         self._test_fn('fp_random', maintains_ref_integrity=False, stable=False, maintains_type=True)
 
+    @unittest.skipIf(common.test_level_lt('checkin'), "Skipping below checkin test level")
     def test_nfp_random(self):
         self._test_fn('fp_random', maintains_ref_integrity=False, stable=False, maintains_type=False)
 
