@@ -885,7 +885,8 @@ class RolesTest(common.TestBase):
             test_role = "list_roles_role1"
             test_role2 = "manageable_test_role"
             test_role3 = "manage_groups_test_role"
-            test_group = "list_roles_group1"
+            test_group = "list_roles_group"
+            test_group_1 = "list_roles_group1"
             test_db = "list_roles_db1"
             test_tbl = "list_roles_tbl1"
 
@@ -910,6 +911,7 @@ class RolesTest(common.TestBase):
                    HAVING ATTRIBUTE IN (%s)
                    TO ROLE %s""" % (test_db, test_tbl, test_attr, test_role),
                 "GRANT ROLE %s TO GROUP %s" % (test_role, test_group),
+                "GRANT ROLE %s TO GROUP %s" % (test_role, test_group_1),
 
                 "DROP ROLE IF EXISTS %s" % (steward_role),
                 "CREATE ROLE %s" % (steward_role),
@@ -964,10 +966,10 @@ class RolesTest(common.TestBase):
                 # Roles with group substring match
                 params = _thrift_api.TListRolesParams()
                 params.return_structured_exprs = True
-                params.group_name_substring = test_group[1:-1]
+                params.group_name_substring = test_group_1[1:-1]
                 roles = list_roles(conn, params).roles
                 if has_access:
-                    assert len(roles) == 1
+                    assert len(roles) >= 1
                     assert contains_role(test_role, roles)
                 else:
                     assert len(roles) == 0
@@ -975,7 +977,7 @@ class RolesTest(common.TestBase):
 
                 params = _thrift_api.TListRolesParams()
                 params.return_structured_exprs = True
-                params.group_name_substring = test_group[1:-1] + "not_exist"
+                params.group_name_substring = test_group_1[1:-1] + "not_exist"
                 roles = list_roles(conn, params).roles
                 assert len(roles) == 0
                 self._verify_structured_exprs(roles)
@@ -1372,6 +1374,66 @@ class RolesTest(common.TestBase):
             assert get_role(conn, as_owner_role).role.is_user_role == True
             assert get_role(conn, steward_role).role.is_user_role == False
 
+    def test_list_roles_groups_filter(self):
+        test_role_1 = "list_roles_groups_filter_role_1"
+        test_role_2 = "list_roles_groups_filter_role_2"
+        test_group_1 = "list_roles_groups_filter_group_1"
+        test_group_2 = "list_roles_groups_filter_group_2"
+        test_group_3 = "list_roles_groups_filter_group_3"
+
+        ctx = common.get_test_context()
+        with common.get_planner(ctx) as conn:
+
+            ddls = [
+                # Create roles
+                "DROP ROLE IF EXISTS %s" % (test_role_1),
+                "CREATE ROLE %s" % (test_role_1),
+                "DROP ROLE IF EXISTS %s" % (test_role_2),
+                "CREATE ROLE %s" % (test_role_2),
+
+                # Assign groups to roles
+                "GRANT ROLE %s TO GROUP %s" % (test_role_1, test_group_1),
+                "GRANT ROLE %s TO GROUP %s" % (test_role_1, test_group_2),
+                "GRANT ROLE %s TO GROUP %s" % (test_role_2, test_group_2),
+            ]
+
+            for ddl in ddls:
+                conn.execute_ddl(ddl)
+
+            # Filter by test_group_1
+            params = _thrift_api.TListRolesParams()
+            params.return_structured_exprs = True
+            params.group_names = [test_group_1]
+            roles = list_roles(conn, params, limit=5).roles
+            assert len(roles) == 1
+            assert contains_role(test_role_1, roles)
+            assert not contains_role(test_role_2, roles)
+
+            # Filter by test_group_2
+            params = _thrift_api.TListRolesParams()
+            params.return_structured_exprs = True
+            params.group_names = [test_group_2]
+            roles = list_roles(conn, params, limit=5).roles
+            assert contains_role(test_role_1, roles)
+            assert contains_role(test_role_2, roles)
+
+            # Filter by test_group_1 and test_group_3
+            params = _thrift_api.TListRolesParams()
+            params.return_structured_exprs = True
+            params.group_names = [test_group_1, test_group_3]
+            roles = list_roles(conn, params, limit=5).roles
+            assert contains_role(test_role_1, roles)
+
+            # Group name substring + groups filters
+            params = _thrift_api.TListRolesParams()
+            params.return_structured_exprs = True
+            params.group_names = [test_group_1]
+            params.group_name_substring = "list_roles_groups_filter_group_"
+            roles = list_roles(conn, params, limit=5).roles
+            assert contains_role(test_role_1, roles)
+            assert not contains_role(test_role_2, roles)
+
+
     def test_expr_ast(self):
         role = "ast_test_role"
         test_attr_ns = "get_role_ns"
