@@ -12,7 +12,9 @@ import numpy
 from okera._thrift_api import TTypeId
 
 from okera.tests import pycerebro_test_common as common
+from okera.tests import create_test_data
 import cerebro_common as cerebro
+from instrumentation import measure_duration
 
 retry_count = 0
 
@@ -407,5 +409,126 @@ class BasicTest(common.TestBase):
             assert indices[0] == 0
             assert indices[1] == 1
 
+    def test_zd1972(self):
+        DB = "test_zd1972"
+        ctx = common.get_test_context()
+        with common.get_planner(ctx) as conn:
+            self._recreate_test_db(conn, DB)
+            conn.execute_ddl('''
+            CREATE EXTERNAL TABLE %s.chase_kyc_event_v001 (
+              `metadata` STRUCT<event_id:STRING,created_date:BIGINT>,
+                `payload` STRUCT<party_key:STRING,customer_type:STRING,cdd:STRUCT<sowFromHighRiskIndustry:ARRAY<STRING>,countriesSignifWealthGen:ARRAY<STRING>,totalAnnualNetIncomeUsd:INT,estTotalNetWorth:INT,isSrcWealthCorroborated:BOOLEAN,srcWealthCorrobPercent:STRING>,spdd:STRUCT<pepHighRisk:BOOLEAN>,add:STRUCT<exceedingCashAggregate:BOOLEAN,exceedingEFTAggregate:BOOLEAN,additionalProdServices:ARRAY<STRING>>,exceptions:ARRAY<STRUCT<exceptionType:STRING>>,risk_profile:STRUCT<riskScore:STRING,risk_level:STRING>>,
+                  `previouspayload` STRUCT<party_key:STRING,customer_type:STRING,cdd:STRUCT<sowFromHighRiskIndustry:ARRAY<STRING>,countriesSignifWealthGen:ARRAY<STRING>,totalAnnualNetIncomeUsd:INT,estTotalNetWorth:INT,isSrcWealthCorroborated:BOOLEAN,srcWealthCorrobPercent:STRING>,spdd:STRUCT<pepHighRisk:BOOLEAN>,add:STRUCT<exceedingCashAggregate:BOOLEAN,exceedingEFTAggregate:BOOLEAN,additionalProdServices:ARRAY<STRING>>,exceptions:ARRAY<STRUCT<exceptionType:STRING>>,risk_profile:STRUCT<riskScore:STRING,risk_level:STRING>>
+                  )
+            STORED AS AVRO
+            LOCATION 's3a://cerebro-test-corey.sunwold/avro-testing/repro/dt=2021-02-25'
+            TBLPROPERTIES ('avro.schema.url'='s3a://cerebro-test-corey.sunwold/avro/example-schema.json')
+            ''' % DB)
+            res = conn.scan_as_json('%s.chase_kyc_event_v001' % DB)
+            self.assertEqual(1000, len(res))
+
+            # Spot check a few enums
+            self.assertEqual('MASS_MARKET', res[10]['previouspayload']['customer_type'])
+            self.assertEqual('UNKNOWN', res[100]['previouspayload']['customer_type'])
+            self.assertEqual('AFFLUENT', res[500]['previouspayload']['customer_type'])
+            self.assertEqual('HIGH_WEALTH', res[900]['previouspayload']['customer_type'])
+
+    def test_parquet_minmax_scanner_filter(self):
+        db = "parquet_minmax_scanner_filter_db"
+        tbl = "parquet_data"
+        non_existent_id = 5802904565645645
+        existing_id = 580389
+        existing_small_int_id = 1
+        non_existent_double_id = 2.5
+        # we expect these queries to take less than 20 seconds
+        max_duration = 20
+        ctx = common.get_test_context()
+        with common.get_planner(ctx) as conn:
+            create_test_data.create_parquet_table_with_stats(conn, db, tbl)
+            # scan with a numeric predicate with the value that does not exist
+            # so that we'd have to scan over the whole thing if the optimization
+            # is not working.
+            m = measure_duration('predicate with non_existent value duration')
+            with m:
+                res = conn.scan_as_json(
+                'select trip_id from %s.%s where trip_id = %s' % (db, tbl, non_existent_id))
+            print(m.duration)
+            print(res)
+            self.assertEqual(0, len(res))
+            assert m.duration < max_duration
+
+            m = measure_duration('predicate with existing value duration')
+            with m:
+                res = conn.scan_as_json(
+                'select trip_id from %s.%s where trip_id = %s' % (db, tbl, existing_id))
+            print(m.duration)
+            print(res)
+            self.assertEqual(1, len(res))
+            assert m.duration < max_duration
+
+            m = measure_duration('predicate with existing small int value duration')
+            with m:
+                res = conn.scan_as_json(
+                'select trip_id from %s.%s where trip_id = %s' % (db, tbl, existing_small_int_id))
+            print(m.duration)
+            print(res)
+            self.assertEqual(1, len(res))
+            assert m.duration < max_duration
+
+            m = measure_duration('predicate with non_existent double value duration')
+            with m:
+                res = conn.scan_as_json(
+                'select trip_id from %s.%s where trip_id = %s' % (db, tbl, non_existent_double_id))
+            print(m.duration)
+            print(res)
+            self.assertEqual(0, len(res))
+            assert m.duration < max_duration
+
+    # TODO: pushdown predicates with non-existing values do not have
+    # consistent performance.
+    @unittest.skip("Perf of predicate pushdown is unpredictable")
+    def test_orc_minmax_scanner_filter(self):
+        db = "orc_minmax_scanner_filter_db"
+        tbl = "orc_data"
+        non_existent_id = 5802904565645645
+        existing_id = 580389
+        existing_small_int_id = 1
+        # we expect these queries to take less than 30 seconds
+        max_duration = 30
+        ctx = common.get_test_context()
+        with common.get_planner(ctx) as conn:
+            create_test_data.create_orc_table_with_stats(conn, db, tbl)
+            # scan with a numeric predicate with the value that does not exist
+            # so that we'd have to scan over the whole thing if the optimization
+            # is not working.
+
+            m = measure_duration('predicate with non_existent value duration')
+            with m:
+                res = conn.scan_as_json(
+                'select trip_id from %s.%s where trip_id = %s' % (db, tbl, non_existent_id))
+            print(m.duration)
+            print(res)
+            self.assertEqual(0, len(res))
+
+            assert m.duration < max_duration
+
+            m = measure_duration('predicate with existing value duration')
+            with m:
+                res = conn.scan_as_json(
+                'select trip_id from %s.%s where trip_id = %s' % (db, tbl, existing_id))
+            print(m.duration)
+            print(res)
+            self.assertEqual(1, len(res))
+            assert m.duration < max_duration
+
+            m = measure_duration('predicate with existing small int value duration')
+            with m:
+                res = conn.scan_as_json(
+                'select trip_id from %s.%s where trip_id = %s' % (db, tbl, existing_small_int_id))
+            print(m.duration)
+            print(res)
+            self.assertEqual(1, len(res))
+            assert m.duration < max_duration
+
 if __name__ == "__main__":
     unittest.main()
